# This file is generated by gen.py
import numpy as np

from kirin.prelude import basic_no_opt


@basic_no_opt
def numpy_passing(x: np.ndarray):
    return x


numpy_passing.code.print()


def test_passing():
    truth = np.arange(3)

    assert numpy_passing(truth) is truth
